import amqp from 'amqplib/callback_api'
import uuid from 'uuid/v4'

const { AMQP_USER, AMQP_PASSWORD, AMQP_HOST, AMQP_PORT, AMQP_VHOST } = process.env

let vhost = ''

if (AMQP_VHOST) {
  vhost = `/${AMQP_VHOST}`
}

const AMQP_CONNECT_URL = `amqp://${AMQP_USER}:${AMQP_PASSWORD}@${AMQP_HOST}:${AMQP_PORT}${vhost}`

export class GlobalEventBus {
  /**
   * @private
   */
  _connection = null
  /**
   * @private
   */
  _channel = null

  /**
   * @private
   * @type {amqp}
   */
  _amqpProvider = null

  /**
   *
   * @type {string}
   * @private
   */
  _amqpConnectUrl = ""

  constructor({
    amqpConnectUrl = AMQP_CONNECT_URL,
    amqpProvider = amqp,
    reconnectInterval = 5000
  }) {
    this._amqpProvider = amqpProvider
    this._amqpConnectUrl = amqpConnectUrl
    this._reconnectInterval = reconnectInterval
  }
  /**
   * @private
   */
  _createChannel() {
    return new Promise((resolve, reject) => {
      this._connection.createChannel((err, channel) => {
        if (err) {
          return reject(err)
        }
        resolve(channel)
      })
    })
  }
  /**
   * @private
   */
  _connect() {
    return new Promise((resolve, reject) => {
      this._amqpProvider.connect(
        this._amqpConnectUrl,
        (err, connection) => {
          if (err) {
            console.error(`[AMQP] Failed connection, reconnecting in ${this._reconnectInterval}: ${err}`)
            setTimeout(this.init, this._reconnectInterval)
          }

          connection.on('error', error => {
            if (error.message !== 'Connection closing') {
              console.error(`[AMQP] connection error: ${error.message}`)
            }
          })

          connection.on('close', () => {
            console.error(`[AMQP] Connection closed, reconnecting in ${this._reconnectInterval}ms`)
            setTimeout(this.init, this._reconnectInterval)
          })

          resolve(connection)
        }
      )
    })
  }
  /**
   * @private
   */
  _assertQueue(type, options) {
    return new Promise((resolve, reject) => {
      this._channel.assertQueue(type, options, (err, q) => {
        if (err) {
          return reject(err)
        }
        resolve(q)
      })
    })
  }

  /**
   * @private
   */
  async _sendEventWithoutConfirm(type, data, options = {}, queueOptions = {}) {
    await this._assertQueue(type, { durable: false, ...queueOptions })
    this._channel.sendToQueue(type, data, { persistent: false, ...options })
  }

  init = async () => {
    console.info(`[AMQP]: Trying create connection`)
    try {
      this._connection = await this._connect()
      this._channel = await this._createChannel()
    } catch (err) {
      throw err
    }
  }

  /**
   * Send event without waiting answer
   * @param type - type o message
   * @param dataObject - object with datas of you send to worker
   * @returns {Promise<void>}
   */
  publishEvent = async (type, dataObject) => {
    const jsonObjectBuffer = new Buffer(JSON.stringify(dataObject))

    await this._sendEventWithoutConfirm(type, jsonObjectBuffer)
  }

  /**
   *
   * @param type - type of message
   * @param handler - callback with received data from channel
   * @returns {void}
   */
  subscribeOnEvent = async (type, handler) => {
    await this._assertQueue(type, { durable: false })
    this._channel.consume(type, async msg => {
      const data = JSON.parse(msg.content.toString())

      await Promise.resolve(handler(data))

      this._channel.ack(msg)
    })
  }
}
